package org.q.app.clipboard;

import java.awt.datatransfer.DataFlavor;
import java.io.File;
import java.io.IOException;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.q.app.vfs.UrlResolver;
import org.q.toolkit.io.UrlTool;
import org.q.toolkit.lang.Str;
import org.q.toolkit.lang.collection.NewCollection;

/**
 * This class defines a single task on or from the system clipboard.
 * 
 * @author Erik Lievaart
 */
public class ClipboardTask {
	/** Flavor of the task. */
	public final static DataFlavor CLIPBOARD_TASK_FLAVOR = new DataFlavor(ClipboardTask.class, "ClipboardTask");

	private final String operation;
	private final String urls;

	/**
	 * Constructor
	 * 
	 * @param operation
	 *            The Q command to invoke when copying or moving the files. Should be "copy" or "move".
	 * @param urls
	 *            These URL's should not be escaped and should be separated by the newline character '\n'.
	 */
	ClipboardTask(final String operation, final String urls) {
		this.operation = operation;
		this.urls = urls;
	}

	/**
	 * Constructor
	 * 
	 * @param operation
	 *            The Q command to invoke when copying or moving the files. Should be "copy" or "move".
	 * @param urls
	 *            These URL's should be escaped.
	 */
	public ClipboardTask(final String operation, final List<String> urls) {
		this(operation, unescapeAndJoinLines(urls));
	}

	/**
	 * Constructor for a copy task
	 */
	ClipboardTask(final List<File> files) {
		this("copy", getFilePaths(files));
	}

	static String getFilePaths(final List<File> files) {
		List<String> paths = NewCollection.list();
		for (File file : files) {
			paths.add(file.getAbsolutePath());
		}
		return StringUtils.join(paths, '\n');
	}

	/**
	 * Get the Q operation to invoke.
	 */
	public String getOperation() {
		return operation;
	}

	String getUrls() {
		return urls;
	}

	static String unescapeAndJoinLines(final List<String> urls) {
		List<String> unescaped = NewCollection.list();
		for (String url : urls) {
			unescaped.add(UrlTool.unescape(url));
		}
		return StringUtils.join(unescaped, '\n');
	}

	/**
	 * List the URL's that are considered to be invalid.
	 */
	public List<String> getInvalidUrls() {
		List<String> invalid = NewCollection.list();

		for (String url : getEscapedUrlList()) {
			try {
				if (!UrlResolver.resolve(url).exists()) {
					invalid.add(Str.sub("url does not exist: %", url));
				}
			} catch (IOException e) {
				invalid.add(Str.sub("Exception on url % =>", url) + e.getMessage());
			}
		}
		return invalid;
	}

	List<File> getFiles() {
		List<File> list = NewCollection.list();
		if (urls == null) {
			return list;
		}

		for (String url : StringUtils.split(urls, '\n')) {
			list.add(new File(UrlTool.getPath(url)));
		}
		return list;
	}

	/**
	 * Get a List of all URL's in this task.
	 * 
	 * @return the escaped URL's in a List.
	 */
	public List<String> getEscapedUrlList() {
		List<String> list = NewCollection.list();
		if (urls == null) {
			return list;
		}

		for (String url : StringUtils.split(urls, '\n')) {
			list.add(UrlTool.escape(url));
		}
		return list;
	}

	/**
	 * Are there any URL's in this task?
	 */
	public boolean isEmpty() {
		return StringUtils.isEmpty(urls);
	}

}
