package org.q.app.engine.concurrent;

import java.util.concurrent.ArrayBlockingQueue;

import org.q.app.engine.impl.PluginRunnable;
import org.q.toolkit.lang.LogTemplate;
import org.q.toolkit.test.AntProperties;

import com.google.inject.Inject;
import com.google.inject.Singleton;

/**
 * This is the central queue in Q. Long running jobs are queued in the
 * CommandQueue and executed one by one. The CommandQueue also has methods for
 * clearing or pausing the queue.
 * 
 * @author Erik Lievaart
 */
@Singleton
public class CommandQueue {

	private final LogTemplate log = new LogTemplate(getClass());

	private final ArrayBlockingQueue<PluginRunnable> queue = new ArrayBlockingQueue<>(100);
	private final CommandQueueScripts scripts;

	@Inject
	CommandQueue(final CommandQueueScripts script) {
		this.scripts = script;

		Thread thread = new Thread(new QueueRunnable());
		thread.setDaemon(false);
		thread.start();
	}

	void invokeLater(final PluginRunnable runnable) {
		queue.add(runnable);
		scripts.add(runnable.getId(), runnable.getLabel());
	}

	private class QueueRunnable implements Runnable {
		private PluginRunnable job;

		@Override
		public void run() {
			while (true) {
				runJobs();
			}
		}

		private void runJobs() {
			try {
				runJobsInQueue();
				Thread.sleep(AntProperties.isTestMode() ? 5 : 100);

			} catch (InterruptedException e) {
				log.error("InterruptedException from QueueRunnable", e);
				throw new RuntimeException("Interrupted", e);
			}
		}

		private void runJobsInQueue() {
			job = queue.poll();

			while (job != null) {
				job.run();

				if (job.isError()) {
					boolean keepGoing = scripts.failed(job.getId(), job.getLabel(), job.getErrorMessage());

					if (!keepGoing) {
						queue.clear();
					}
				}
				scripts.done(job.getId(), job.getLabel());
				job = queue.poll();
			}
		}
	}

}
