package org.q.app.engine.context;

import java.io.IOException;
import java.util.Collections;
import java.util.Map;

import org.q.app.config.ConfigurableResources;
import org.q.toolkit.io.RuntimeIOException;
import org.q.toolkit.lang.Check;
import org.q.toolkit.lang.collection.NewCollection;

/**
 * Base implementation for storing and retrieving variables.
 * 
 * @author Erik Lievaart
 */
public abstract class AbstractUserContext implements VariableContext {

	private final Map<String, String> memory = NewCollection.concurrentMap();
	private final Map<String, String> permanent;

	private final ConfigurableResources resources;
	private final String path;

	AbstractUserContext(final ConfigurableResources classpath, final String path) {
		this.resources = classpath;
		this.path = path;

		permanent = classpath.loadIniFile(path);
		memory.putAll(permanent);
	}

	@Override
	public Map<String, String> getVariables() {
		return Collections.unmodifiableMap(memory);
	}

	@Override
	public void setVariable(final String name, final String value) {
		Check.notBlank(value);
		memory.put(name, value);
	}

	@Override
	public void setPermanentVariable(final String name, final String value) {
		setVariable(name, value);
		permanent.put(name, value);
		savePermanentVariables(permanent);
	}

	void savePermanentVariables(final Map<String, String> variables) {
		try {
			resources.storeIniFile(path, variables);
		} catch (IOException e) {
			throw new RuntimeIOException("Unable to save Variables; " + e.getMessage(), e);
		}
	}

	@Override
	public String getVariable(final String name) {
		return memory.get(name);
	}

	@Override
	public String getVariable(final String name, final String ifAbsent) {
		return getVariable(name) == null ? ifAbsent : getVariable(name);
	}
}
