package org.q.app.engine.meta;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang3.ArrayUtils;
import org.q.app.contract.ThreadPolicy;
import org.q.app.engine.parse.ShellCommand;
import org.q.toolkit.lang.Check;
import org.q.toolkit.lang.Contract;
import org.q.toolkit.lang.collection.MapTool;
import org.q.toolkit.lang.collection.NewCollection;
import org.q.toolkit.lang.concurrent.Immutable;

/**
 * This class contains all available meta data for a single plugin. This class is immutable.
 * 
 * @author Erik Lievaart
 */
@Immutable
@Contract
public class CommandMetadata implements Comparable<CommandMetadata> {

	private final CommandNameMetadata name;
	private final InvocationMetadata invocation;
	private final Map<Character, FlagMetadata> flags;
	private final List<FlagGroupMetadata> groups;

	CommandMetadata(final CommandNameMetadata name, final InvocationMetadata invocation,
			final List<FlagGroupMetadata> groups) {

		this.invocation = invocation;
		this.name = name;
		this.groups = groups;
		this.flags = createFlagMap(groups);

		checkAssertionExceptions();
	}

	private Map<Character, FlagMetadata> createFlagMap(final List<FlagGroupMetadata> metadata) {
		Map<Character, FlagMetadata> map = NewCollection.map();
		for (FlagGroupMetadata group : metadata) {
			for (FlagMetadata item : group.getAllFlags()) {
				map.put(item.getFlag(), item);
			}
		}
		return MapTool.unmodifiableCopy(map);
	}

	private void checkAssertionExceptions() {
		Check.notBlank(name.getQName(), "Command must have a name.");
		Check.notNull(name.getLiteral(), "Command must have a class literal.");
		Check.isTrue(invocation.isInvocable() || invocation.getSwallows(), "Command must run or swallow: %", name);
	}

	@SuppressWarnings("unchecked")
	static CommandMetadata key(final String cmd) {
		CommandNameMetadata cmn = new CommandNameMetadata(cmd, CommandMetadata.class.getName());
		InvocationMetadata im = new InvocationMetadata(ThreadPolicy.SWING, true, false, false);

		return new CommandMetadata(cmn, im, Collections.EMPTY_LIST);
	}

	/**
	 * Get the meta data of the name of the Command.
	 * 
	 * @return information about the name.
	 */
	public CommandNameMetadata getName() {
		return name;
	}

	FlagMetadata[] getFlags() {
		return flags.values().toArray(new FlagMetadata[] {});
	}

	/**
	 * Returns the flags specified in the {@link ShellCommand} complemented with any flags that are on by default.
	 * 
	 * @return the letter of the flags only.
	 */
	public char[] getFlagsOrDefaults(final ShellCommand command) {
		List<Character> result = NewCollection.list();
		Set<String> groupsSpecified = NewCollection.set();

		for (char c : command.getFlags()) {
			result.add(c);
			groupsSpecified.add(getGroup(c));
		}
		for (FlagGroupMetadata group : groups) {
			if (!groupsSpecified.contains(group.getName()) && group.hasPrimaryFlag()) {
				result.add(group.getPrimaryFlag());
			}
		}
		return ArrayUtils.toPrimitive(result.toArray(new Character[] {}));
	}

	String getGroup(final char f) {
		for (FlagGroupMetadata group : groups) {
			if (group.containsFlag(f)) {
				return group.getName();
			}
		}
		return null;
	}

	/**
	 * List the flags available for this plugin.
	 * 
	 * @return the letter of the flags only.
	 */
	public char[] getCharacterFlags() {
		char[] chars = ArrayUtils.toPrimitive(this.flags.keySet().toArray(new Character[] {}));
		Arrays.sort(chars);
		return chars;
	}

	/**
	 * Return the meta data of one of this command's flags.
	 * 
	 * @param flag
	 *            Character identifying the flag.
	 * @return The FlagMetadata.
	 */
	public FlagMetadata getFlagMetadata(final char flag) {
		return flags.get(flag);
	}

	/**
	 * Compares plugins by their name.
	 */
	@Override
	public int compareTo(final CommandMetadata other) {
		return name.getQName().compareTo(other.name.getQName());
	}

	boolean isValidateSwallowed() {
		return invocation.isValidateSwallowed();
	}

	/**
	 * Get the meta data for invoking the command.
	 */
	public InvocationMetadata getInvocationMetadata() {
		return invocation;
	}

	/**
	 * Return the ThreadPolicy for the ShellCommand.
	 * 
	 * @return the policy to use when executing this command.
	 */
	public ThreadPolicy getThreadPolicy() {
		return invocation.getThreadPolicy();
	}

	/**
	 * Returns whether or not a flag is primary.
	 */
	public boolean isPrimary(final char flag) {
		for (FlagGroupMetadata group : this.groups) {
			if (group.hasPrimaryFlag() && group.getPrimaryFlag() == flag) {
				return true;
			}
		}
		return false;
	}

}
