package org.q.app.engine.meta;

import java.lang.reflect.Method;
import java.lang.reflect.Type;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.q.app.contract.Invocable;
import org.q.app.contract.Swallows;
import org.q.app.contract.ThreadPolicy;
import org.q.app.contract.annotation.Concurrency;
import org.q.app.contract.annotation.Doc;
import org.q.app.contract.annotation.Flag;
import org.q.app.contract.annotation.ValidateSwallowed;
import org.q.app.engine.convert.BasicConvertersProvider;
import org.q.app.engine.parse.ShellParseException;
import org.q.app.engine.parse.ShellParser;
import org.q.toolkit.convert.Converters;
import org.q.toolkit.lang.Check;
import org.q.toolkit.lang.collection.NewCollection;
import org.q.toolkit.lang.concurrent.Stateless;
import org.q.toolkit.reflect.GenericsTool;
import org.q.toolkit.reflect.ReflectException;
import org.q.toolkit.reflect.annotations.AnnotatedMethod;
import org.q.toolkit.reflect.annotations.AnnotationTool;

import com.google.inject.Inject;

@Stateless
class ShellCommandIntrospector {

	@Inject
	private Converters converters;

	private Converters basicConverters;

	@Inject
	void init(final BasicConvertersProvider basicConvertersProvider) {
		basicConverters = basicConvertersProvider.get();
	}

	public MetadataCollection load(final Map<String, String> mapping) {
		List<CommandMetadata> metadata = NewCollection.list();

		for (String command : mapping.keySet()) {
			metadata.add(introspect(new CommandMetadataB(command, mapping.get(command))));
		}
		return new MetadataCollection(metadata);
	}

	CommandMetadata introspect(final CommandMetadataB builder) {
		Class<?> literal = builder.getName().getLiteral();

		builder.setFlagGroupMetadata(loadFlags(literal));
		enforceFlagsUnique(builder.getFlagGroupMetadata(), literal);

		introspectInvocationData(builder, literal);
		ReflectException.on(!builder.isExecutable(), "Plugin must implement Swallows or Invocable: %", literal);

		CommandMetadata plugin = builder.create();
		validateDocumented(literal, plugin);
		return plugin;
	}

	void validateDocumented(final Class<?> clazz, final CommandMetadata plugin) {
		ReflectException.unless(AnnotationTool.literalHasAnnotation(clazz, Doc.class), "No class @Doc: $", clazz);

		for (FlagMetadata flag : plugin.getFlags()) {
			boolean documented = AnnotationTool.methodHasAnnotation(clazz, flag.getName(), Doc.class);
			ReflectException.unless(documented, "Missing @Doc for @Flag % on: %", flag.getName(), clazz);
		}
	}

	private static void enforceFlagsUnique(final List<FlagGroupMetadata> list, final Class<?> literal) {
		Set<Character> character = NewCollection.set();

		for (FlagGroupMetadata group : list) {
			for (FlagMetadata item : group.getAllFlags()) {
				boolean duplicate = !character.add(item.getFlag());
				ReflectException.on(duplicate, "Duplicate @Flag % => % on %", item.getFlag(), item.getName(), literal);
			}
		}
	}

	List<FlagGroupMetadata> loadFlags(final Class<?> clazz) {
		FlagGroupMetadataB builder = new FlagGroupMetadataB(clazz.getName());

		for (AnnotatedMethod<Flag> method : AnnotationTool.getMethodsAnnotatedWith(clazz, Flag.class)) {
			validateFlag(method);

			Flag annotation = method.getAnnotation();
			FlagMetadata fm = new FlagMetadata(method.getName(), loadFlagValues(method));
			builder.addFlag(StringUtils.trim(annotation.group()), fm, annotation.primary());
		}
		return builder.createList();
	}

	static List<ArgumentMetadata> loadFlagValues(final AnnotatedMethod<Flag> annotated) {

		Method member = annotated.getMember();
		String[] values = annotated.getAnnotation().values();
		Class<?>[] types = member.getParameterTypes();
		ReflectException.on(values.length != types.length, "% values required for member %", types.length, member);

		try {
			return parseFlagValues(values);
		} catch (ShellParseException e) {
			throw new ReflectException(member + e.getMessage(), e);
		}
	}

	private static List<ArgumentMetadata> parseFlagValues(final String[] values) throws ShellParseException {
		List<ArgumentMetadata> result = NewCollection.list();

		for (int i = 0; i < values.length; i++) {
			result.add(new ArgumentMetadata(ShellParser.parseArgument(values[i])));
		}
		return result;
	}

	private void validateFlag(final AnnotatedMethod<Flag> annotated) {
		Class<?>[] types = annotated.getMember().getParameterTypes();
		Type[] generics = annotated.getMember().getGenericParameterTypes();

		for (int i = 0; i < types.length; i++) {
			boolean convertible = converters.isConvertible(types[i]);
			ReflectException.unless(convertible, "Unsupported %; Supported @Flag: %", types[i], converters.getTypes());
			verifyGenerics(annotated, types[i], generics[i]);
		}
	}

	private void verifyGenerics(final AnnotatedMethod<Flag> annotated, final Class<?> literal, final Type generics) {
		if (literal == List.class) {
			Type type = GenericsTool.getGenericType(generics);
			boolean isConvertible = basicConverters.isConvertible(type);
			ReflectException.unless(isConvertible, "Cannot convert %; only: %", type, basicConverters.getTypes());
		}
		if (literal == Map.class) {
			boolean invalid = !GenericsTool.hasGenericTypes(generics, String.class, String.class);
			ReflectException.on(invalid, "Maps should have Generic types 'String', 'String': %", annotated);
		}
	}

	private static void introspectInvocationData(final CommandMetadataB builder, final Class<?> literal) {
		boolean swallows = willSwallow(literal);

		builder.setInvocable(isInvocable(literal));
		builder.setSwallows(swallows);
		builder.setValidate(isValidateSwallowed(literal));
		builder.setThreadPolicy(getThreadPolicy(literal));

		Check.isTrue(builder.isValidationLegal(), "Remove @ValidateSwallowed or implement Swallows: %", literal);
		if (swallows) {
			validateSwallowedDocumented(literal);
		}
	}

	static void validateSwallowedDocumented(final Class<?> literal) {
		for (AnnotatedMethod<Doc> method : AnnotationTool.getMethodsAnnotatedWith(literal, Doc.class)) {
			if (method.getName().equals("invoke")) {
				Class<?>[] parameters = method.getMember().getParameterTypes();
				if (parameters.length == 1 && parameters[0].equals(String.class)) {
					return;
				}
			}
		}
		throw new ReflectException("Missing @Doc annotation on invoke(String) %", literal);
	}

	static ThreadPolicy getThreadPolicy(final Class<?> literal) {
		Concurrency annotation = AnnotationTool.getLiteralAnnotation(literal, Concurrency.class);
		return annotation == null ? ThreadPolicy.SWING : annotation.value();
	}

	static boolean isValidateSwallowed(final Class<?> clazz) {
		return AnnotationTool.literalHasAnnotation(clazz, ValidateSwallowed.class);
	}

	static boolean willSwallow(final Class<?> clazz) {
		return ArrayUtils.contains(clazz.getInterfaces(), Swallows.class);
	}

	static boolean isInvocable(final Class<?> clazz) {
		return ArrayUtils.contains(clazz.getInterfaces(), Invocable.class);
	}

}
