package org.q.app.ui.render.terminal;

import java.util.List;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.q.app.config.ResourcePaths;
import org.q.app.engine.script.ScriptConfig;
import org.q.app.engine.script.ScriptInvoker;
import org.q.app.general.DetailMessage;
import org.q.app.general.ValueHolder;
import org.q.toolkit.lang.collection.MapTool;
import org.q.toolkit.lang.collection.NewCollection;
import org.q.toolkit.swing.SwingThread;

import com.google.inject.Inject;
import com.google.inject.Singleton;

/**
 * This class invokes the {@link ResourcePaths#UI_TERMINAL_OUTPUT_SCRIPT} when output is sent to the terminal. Since the
 * script for adding a line to the terminal is in some cases invoked a ridiculous number of times, this class buffers
 * the messages for updating the UI for efficiency. This class is a best effort attempt, some messages might be lost.
 * 
 * @author Erik Lievaart
 */
@Singleton
class TerminalScript {

	private final ScriptInvoker runner;
	private final List<ValueHolder> messages = NewCollection.list();
	private final ScheduledThreadPoolExecutor scheduler = new ScheduledThreadPoolExecutor(1);

	@Inject
	TerminalScript(final ScriptInvoker runner) {
		ScriptConfig config = new ScriptConfig(MapTool.of("messages", messages));
		config.setDefaultProperties(false);
		runner.createEngine(config);
		this.runner = runner;
	}

	synchronized void addMessage(final DetailMessage commandMessage) {
		messages.add(new ValueHolder(commandMessage, commandMessage.getMessage()));
		if (scheduler.getQueue().size() < 1) { // I wonder if this method is reliable
			scheduler.schedule(new ScriptRunnable(), 100, TimeUnit.MILLISECONDS);
		}
	}

	private synchronized void updateTerminal() {
		if (!messages.isEmpty()) {
			SwingThread.invokeLater(new Runnable() {
				@Override
				public void run() {
					runner.runResource(ResourcePaths.UI_TERMINAL_OUTPUT_SCRIPT);
				}
			});
		}
	}

	private class ScriptRunnable implements Runnable {
		@Override
		public void run() {
			updateTerminal();
		}
	}
}
