package org.q.app.vfs;

import java.io.File;

import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.commons.vfs.FileObject;
import org.apache.commons.vfs.FileSystemException;
import org.apache.commons.vfs.provider.local.LocalFile;
import org.q.toolkit.io.RuntimeIOException;
import org.q.toolkit.io.UrlTool;
import org.q.toolkit.lang.LogTemplate;
import org.q.toolkit.lang.Str;

/**
 * Although the CommonsFile class is immutable, the underlying filesystem is
 * not.
 * 
 * @author Erik Lievaart
 */
class CommonsFile implements VirtualFile {

	private final FileObject file;

	CommonsFile(final FileObject object) {
		file = object;
	}

	@Override
	public boolean exists() {
		String path = getAbsolutePath();
		try {
			boolean exists = file.exists() || isLocal() && getLocalFile().exists();
			if (!exists && !path.equals(path.trim())) {
				new LogTemplate(getClass()).warn("file path is not the same after trimming: %", path);
			}
			return exists;
		} catch (FileSystemException e) {
			throw new RuntimeIOException("Unable to determine if file exists: " + Str.quote(path), e);
		}
	}

	@Override
	public boolean isDirectory() {
		try {
			return file.getType().hasChildren();
		} catch (FileSystemException e) {
			throw new RuntimeIOException("Unable to determine directory type: " + Str.quote(getAbsolutePath()), e);
		}
	}

	@Override
	public boolean isFile() {
		try {
			return file.getType().hasContent();
		} catch (FileSystemException e) {
			throw new RuntimeIOException("Unable to determine if this is a file: " + Str.quote(getAbsolutePath()), e);
		}
	}

	@Override
	public boolean isHidden() {
		try {
			return file.isHidden();
		} catch (FileSystemException e) {
			throw new RuntimeIOException("Unable to determine if file is hidden: " + Str.quote(getAbsolutePath()), e);
		}
	}

	@Override
	public VirtualFile[] listFiles() {
		try {
			FileObject[] children = file.getChildren() == null ? new FileObject[] {} : file.getChildren();
			VirtualFile[] result = new VirtualFile[children.length];
			for (int i = 0; i < children.length; i++) {
				result[i] = new CommonsFile(children[i]);
			}
			return result;

		} catch (FileSystemException e) {
			throw new RuntimeIOException("Unable to retrieve children: " + Str.quote(getAbsolutePath()), e);
		}
	}

	@Override
	public VirtualFile getParentFile() {
		try {
			FileObject parent = file.getParent();
			if (parent == null) {
				return null;
			}
			return new CommonsFile(parent);
		} catch (FileSystemException e) {
			throw new RuntimeIOException("Unable to retrieve parent: " + Str.quote(getAbsolutePath()), e);
		}
	}

	@Override
	public boolean hasParent() {
		try {
			return file.getParent() != null;
		} catch (FileSystemException e) {
			throw new RuntimeIOException(Str.sub("Unable to determine if file has parent: %", getAbsolutePath()), e);
		}
	}

	@Override
	public boolean isLocal() {
		return file instanceof LocalFile;
	}

	@Override
	public File getLocalFile() {
		return new File(getAbsolutePath());
	}

	@Override
	public FileObject getFileObject() {
		return file;
	}

	@Override
	public VirtualFile resolveFile(final String name) {
		try {
			return new CommonsFile(file.resolveFile(name));
		} catch (FileSystemException e) {
			throw new RuntimeIOException(Str.sub("Unable to resolve child: % => %", getAbsolutePath(), name), e);
		}
	}

	@Override
	public int hashCode() {
		return getUrlEscaped().hashCode();
	}

	@Override
	public boolean equals(final Object obj) {
		if (!(obj instanceof VirtualFile)) {
			return false;
		}
		VirtualFile other = (VirtualFile) obj;
		return getUrlEscaped().equals(other.getUrlEscaped());
	}

	@Override
	public String getUrlEscaped() {
		// escape(unescape()) was done deliberately, guarded by unit tests
		return UrlTool.escape(UrlTool.unescape(file.getName().getFriendlyURI()));
	}

	@Override
	public String getUrl() {
		return UrlTool.unescape(getUrlEscaped());
	}

	@Override
	public String getAbsolutePath() {
		return UrlTool.getPath(getUrl());
	}

	@Override
	public String getName() {
		return UrlTool.getName(getUrl());
	}

	@Override
	public String getNameEscaped() {
		return UrlTool.getName(getUrlEscaped());
	}

	@Override
	public String getExtension() {
		return UrlTool.getExtension(getName());
	}

	@Override
	public String getBaseName() {
		return UrlTool.getBaseName(getName());
	}

	@Override
	public long getSize() {
		try {
			return getFileObject().getContent().getSize();
		} catch (FileSystemException e) {
			throw new RuntimeIOException(e);
		}
	}

	@Override
	public String toString() {
		return new ToStringBuilder(this).append(getUrlEscaped()).toString();
	}

}
