package org.q.plugins.ant;

import java.io.File;
import java.util.Collections;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.DefaultLogger;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.ProjectHelper;
import org.q.toolkit.lang.concurrent.Prototype;

/**
 * This is a utility class for calling Ant from Java. It is designed to have no dependencies on Q.
 * 
 * @author Erik Lievaart
 */
@Prototype
class AntProject {

	/** Location of the build file. */
	private final String buildFile;
	private final Map<String, String> properties;

	/**
	 * Constructor.
	 * 
	 * @param file
	 *            buildFile this AntProject should execute.
	 */
	AntProject(final String file) {
		this.buildFile = file;
		this.properties = Collections.emptyMap();
	}

	/**
	 * Constructor.
	 * 
	 * @param file
	 *            buildFile this AntProject should execute.
	 * @param properties
	 *            the build properties.
	 */
	AntProject(final String file, final Map<String, String> properties) {
		this.buildFile = file;
		this.properties = properties;
	}

	void runTarget(final String target) {
		runTarget(createProject(), target);
	}

	private void runTarget(final Project project, final String target) {
		try {
			project.executeTarget(target);
			project.fireBuildFinished(null);

		} catch (BuildException e) {
			project.fireBuildFinished(e);
			throw new RuntimeException(e);
		}
	}

	private Project createProject() {
		File file = new File(buildFile);
		Project project = new Project();
		configureLogger(project);

		project.setNewProperty("ant.file", file.getAbsolutePath());
		for (Entry<String, String> entry : properties.entrySet()) {
			project.setNewProperty(entry.getKey(), entry.getValue());
		}
		project.fireBuildStarted();
		project.init();
		ProjectHelper helper = ProjectHelper.getProjectHelper();
		project.addReference("ant.projectHelper", helper);
		helper.parse(project, file);
		return project;
	}

	/**
	 * Configures the logger for a project.
	 * 
	 * @param p
	 *            If this method is not called for p, the project runs or fails silently.
	 */
	private void configureLogger(final Project p) {
		DefaultLogger consoleLogger = new DefaultLogger();
		consoleLogger.setErrorPrintStream(System.err);
		consoleLogger.setOutputPrintStream(System.out);
		consoleLogger.setMessageOutputLevel(Project.MSG_INFO);
		p.addBuildListener(consoleLogger);
	}

}
