package org.q.toolkit.lang.image;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.image.BufferedImage;

public class BoundedGraphics {

	private final Bounds bounds;
	private final Graphics g;

	public BoundedGraphics(Bounds bounds, Graphics g) {
		this.bounds = bounds;
		this.g = g;
	}

	public BoundedGraphics subBounds(int x, int y, int width, int height) {
		assertInBounds(x, y, width, height);
		return new BoundedGraphics(new Bounds(x + bounds.getX(), y + bounds.getY(), width, height), g);
	}

	public void fillOval(Color color, int x, int y, int width, int height) {
		g.setColor(color);
		assertInBounds(x, y, width, height);
		g.fillOval(x + bounds.getX(), y + bounds.getY(), width, height);
	}

	public void drawRect(Color color, int x, int y, int width, int height) {
		g.setColor(color);
		assertInBounds(x, y, width, height);
		g.drawRect(x + bounds.getX(), y + bounds.getY(), width - 1, height - 1);
	}

	public void fillRect(Color color, int x, int y, int width, int height) {
		g.setColor(color);
		assertInBounds(x, y, width, height);
		g.fillRect(x + bounds.getX(), y + bounds.getY(), width, height);
	}

	public void drawLine(Color color, int x1, int y1, int x2, int y2) {
		assertInBounds(x1, y1, 0, 0);
		assertInBounds(x2, y2, 0, 0);

		g.setColor(color);
		g.drawLine(x1 + bounds.getX(), y1 + bounds.getY(), x2 + bounds.getX(), y2 + bounds.getY());
	}

	public void drawVerticalLine(Color color, int xTop, int yTop, int height) {
		drawLine(color, xTop, yTop, xTop, yTop + height);
	}

	public Bounds getBounds() {
		return bounds;
	}

	public void drawPixel(Color color, int x, int y) {
		fillRect(color, x, y, 1, 1);
	}

	public void drawImage(BufferedImage img) {
		drawImage(img, bounds.getX(), bounds.getY());
	}

	public void drawImage(BufferedImage img, int x, int y) {
		assertInBounds(x, y, img.getWidth(), img.getHeight());
		g.drawImage(img, x + bounds.getX(), y + bounds.getY(), img.getWidth(), img.getHeight(), null);
	}

	public void drawImage(BufferedImage img, int x, int y, int width, int height) {
		assertInBounds(x, y, width, height);
		g.drawImage(img, x + bounds.getX(), y + bounds.getY(), width, height, null);
	}

	public void drawImageCentered(BufferedImage img) {
		int xOffset = (bounds.getWidth() - img.getWidth()) / 2;
		int yOffset = (bounds.getHeight() - img.getHeight()) / 2;
		drawImage(img, xOffset, yOffset);
	}

	public void drawString(Color c, Font font, String value, int x, int y) {
		g.setColor(c);
		g.setFont(font);

		int width = g.getFontMetrics().stringWidth(value);
		int height = g.getFontMetrics().getHeight();
		assertInBounds(x, y - font.getSize(), width, height);

		g.drawString(value, x + bounds.getX(), y + bounds.getY());
	}

	private void assertInBounds(int x, int y, int width, int height) {
		assert x >= 0;
		assert y >= 0;
		assert x + width <= bounds.getWidth() : x + width + " > " + bounds.getWidth();
		assert y + height <= bounds.getHeight() : y + height + " > " + bounds.getHeight();
	}

}
