package org.q.toolkit.reflect;

import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import org.q.toolkit.lang.Str;
import org.q.toolkit.lang.collection.NewCollection;
import org.q.toolkit.lang.collection.NullPolicy;
import org.q.toolkit.reflect.method.PropertyWrapper;

/**
 * Utility class for working with Java bean properties.
 *
 * @author Erik Lievaart
 */
public class PropertyTool {
	private PropertyTool() {
	}

	static Map<String, PropertyWrapper> introspectProperties(final Class<?> literal) {
		Map<String, PropertyWrapper> properties = NewCollection.concurrentMap();
		try {
			for (PropertyDescriptor descriptor : Introspector.getBeanInfo(literal).getPropertyDescriptors()) {
				properties.put(descriptor.getName(), new PropertyWrapper(descriptor));
			}
		} catch (IntrospectionException e) {
			throw new ReflectException(Str.sub("Unable to get descriptors for %", literal), e);
		}
		return properties;
	}

	/**
	 * Filter the property wrappers and return only those properties which have an accessible setter.
	 */
	public static List<PropertyWrapper> getWritableProperties(final Collection<PropertyWrapper> properties) {
		List<PropertyWrapper> result = NewCollection.list(NullPolicy.REJECT);
		for (PropertyWrapper property : properties) {
			if (property.isWritable()) {
				result.add(property);
			}
		}
		return result;
	}

	/**
	 * Return a Map containing all the bean properties for a literal.
	 */
	public static Map<String, PropertyWrapper> getPropertyMap(final String clazz) {
		return LiteralTool.wrap(clazz).getProperties();
	}

	/**
	 * Return a Map containing all the bean properties for a literal.
	 */
	public static Map<String, PropertyWrapper> getPropertyMap(final Class<?> clazz) {
		return LiteralTool.wrap(clazz).getProperties();
	}

	/**
	 * Return the named property for a literal.
	 */
	public static PropertyWrapper getProperty(final Class<?> clazz, final String name) {
		return LiteralTool.wrap(clazz).getProperty(name);
	}

	/**
	 * Return a Collection containing all the bean properties for a literal.
	 */
	public static Collection<PropertyWrapper> getProperties(final String clazz) {
		return LiteralTool.wrap(clazz).getProperties().values();
	}

	/**
	 * Return a Collection containing all the bean properties for a literal.
	 */
	public static Collection<PropertyWrapper> getProperties(final Class<?> clazz) {
		return LiteralTool.wrap(clazz).getProperties().values();
	}

	/**
	 * Return a Collection containing all the writable bean properties for a literal.
	 */
	public static Collection<PropertyWrapper> getWritableProperties(final Class<?> clazz) {
		return getWritableProperties(getProperties(clazz));
	}

}
