package org.q.toolkit.runtime;

import java.util.ArrayList;
import java.util.List;

import org.q.toolkit.lang.Check;
import org.q.toolkit.lang.StringEscape;
import org.q.toolkit.lang.collection.ListTool;
import org.q.toolkit.lang.collection.MapTool;
import org.q.toolkit.lang.concurrent.Immutable;
import org.q.toolkit.lang.pattern.PatternTool;

/**
 * A command that can be invoked on the native shell.
 * 
 * @author Erik Lievaart
 */
@Immutable
public class Command {

	/** Replace escape characters: '\b' & '\w' => '\' & ' '. */
	private static final StringEscape COMMAND_LINE = new StringEscape(MapTool.of('b', '\\', 'w', ' '));

	private final List<String> cmdarray = new ArrayList<String>();

	private Command(final String... cmd) {
		ListTool.addAll(cmdarray, cmd);
	}

	/**
	 * Create a new command with the supplied arguments.
	 */
	public Command(final String command, final String... params) {
		cmdarray.add(command);
		ListTool.addAll(cmdarray, params);
	}

	/**
	 * Create a command from a String. The line will be split on whitespace and the first word will be the command
	 * invoked. All following words will be arguments to the command. Whitespace and backslashes in arguments must be
	 * escaped using \w and \b respectively.
	 */
	public static Command from(final String line) {
		return new Command(splitCommandLine(line));
	}

	/**
	 * Escape whitespace characters and backslashes in an argument.
	 */
	public static String escape(final String input) {
		return COMMAND_LINE.escape(input);
	}

	static String[] splitCommandLine(final String line) {
		Check.notBlank(line);
		return COMMAND_LINE.unescape(PatternTool.split("\\s", line.trim()));
	}

	/**
	 * Returns the command and its arguments in order.
	 */
	public String[] cmd() {
		return cmdarray.size() == 0 ? null : cmdarray.toArray(new String[0]);
	}

	@Override
	public String toString() {
		return cmdarray.toString();
	}

}
