package org.q.app.vfs;

import java.io.IOException;
import java.util.Stack;
import java.util.TreeSet;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.vfs.FileObject;
import org.q.toolkit.io.RuntimeIOException;
import org.q.toolkit.io.UrlTool;
import org.q.toolkit.lang.LogTemplate;
import org.q.toolkit.lang.collection.NewCollection;

class FuzzyUrlResolver {

	private final LogTemplate log = new LogTemplate(getClass());
	private final Stack<String> path = NewCollection.stack();
	private VirtualFile found = null;

	private FuzzyUrlResolver(final String url, final FileObject base) throws IOException {
		resolveExistingParent(url, base);
	}

	private void resolveExistingParent(final String url, final FileObject base) throws IOException {
		String current = preProcessFuzzyUrl(url);

		while (current != null) {
			found = UrlResolver.resolve(current, base);
			if (found.exists() && found.isDirectory()) {
				break;
			}
			if (!base.exists()) {
				resolveExistingParent(".", base.getParent());
				return;
			}
			path.push(UrlTool.getName(current));
			current = UrlTool.removeSlash(UrlTool.getParent(current));
		}
		RuntimeIOException.unless(found.exists(), "unable to resolve path %", url);
	}

	static String preProcessFuzzyUrl(final String url) {
		boolean isUrl = StringUtils.contains(url, ':');

		String slashes = url.replaceAll("\\\\", "/");
		String path = isUrl ? UrlTool.getPath(slashes) : slashes;
		return StringUtils.removeEnd(slashes, path) + path.replaceAll("/(?=/)", "/*");
	}

	public static VirtualFile resolveFuzzy(final String url, final FileObject base) throws IOException {
		return new FuzzyUrlResolver(url, base).resolveChildFuzzy();
	}

	private VirtualFile resolveChildFuzzy() throws IOException {
		log.trace("resolving % in %", path, found);
		while (!path.isEmpty()) {
			String name = UrlTool.removeSlash(path.pop());
			VirtualFile[] children = VirtualFiles.getDirsAlphabetically(found);
			TreeSet<String> names = VirtualFiles.getNamesAlphabetically(children);

			log.trace("looking for % in %", name, names);
			String match = new WildcardResolver(name, names).resolve();
			if (match == null) {
				log.info("Could not find child % in %", name, names);
				return found;
			}
			found = VirtualFiles.findNamedChild(children, match);
		}
		log.debug("Returning match % for path %", found, path);
		return found;
	}

}
